@@ -10,7 +10,7 @@ import unicodedata
 import logging
 import gzip
 
-VERSION = "0.3.0"
+VERSION = "0.4.0"
 
 
 class Dat2Html:
@@ -71,6 +71,66 @@ class Dat2Html:
         output += self.template_footer % params
         return output
 
+    def convert_file(self, input_file, output_dir):
+        filename_noext = re.sub("\.dat\.gz$|\.dat$", "",
+                                os.path.basename(input_file))
+        lines = open_file(input_file).readlines()
+        output = self.convert(lines, filename_noext)
+
+        if output is None:
+            logging.error("Could not parse file: %s" % input_file)
+            return False
+
+        output_file = os.path.join(output_dir, filename_noext + ".html")
+        if self.template_dir == "*text*":
+            output_file = os.path.join(output_dir, filename_noext + ".txt")
+        if os.path.exists(output_file):
+            logging.warning("%s already exists. Overwriting ..." %
+                            output_file)
+
+        logging.info("Generating %s" % output_file)
+        try:
+            if output_dir == "-":
+                output_file = "stdout"
+                sys.stdout.write(output)
+            else:
+                f = open(output_file, "w")
+                f.write(output)
+                f.close()
+        except:
+            logging.error("Failed to write file: %s" % output_file)
+            return False
+
+        return True
+
+    def convert_files(self, input_files, output_dir,
+                      index=False, subject=False):
+        filenames = get_filenames(input_files)
+
+        if len(filenames) <= 0:
+            logging.error("No input files")
+            sys.exit(2)
+
+        if output_dir != "-" and not os.path.exists(output_dir):
+            logging.info("Creating directory ...")
+            try:
+                os.makedirs(output_dir)
+            except OSError, (errorno, strerror):
+                logging.error("Could not create %s: %s" %
+                              (output_dir, strerror))
+                sys.exit(2)
+        if output_dir != "-" and not os.access(output_dir, os.W_OK):
+            logging.error("Could not open %s: permission denied" % output_dir)
+            sys.exit(2)
+
+        for filename in filenames:
+            self.convert_file(filename, output_dir)
+
+        if index:
+            make_index(filenames, output_dir)
+        if subject:
+            make_subject(filenames, output_dir)
+
     def template_exists(self):
         if self.template_dir is None:
             return False
@@ -220,11 +280,14 @@ class Dat2Html:
             (prefix, unicodedata.normalize("NFKC", x.group(2).decode("cp932"))
              .encode("cp932"), x.group(1), x.group(2)), message)
 
-        p = re.compile(r'([^\"]|^)(https?|ftp)(://[\w:;/.?%\#&=+-~!]+)')
-        message = p.sub(r'\1<a href="\2\3" target="_blank">\2\3</a>', message)
-
-        p = re.compile(r'([^h]|^)(ttps?)(://[\w:;/.?%\#&=+-~!]+)')
-        message = p.sub(r'\1<a href="h\2\3" target="_blank">\2\3</a>', message)
+        p = re.compile(r'([^\"h]|^)(h?ttps?|ftp)(://[\w:;/.?%\#&=+-~!*@$]+)')
+        correct_scheme = lambda x: 'http' if x == 'ttp' else x
+        message = p.sub(
+            lambda x: '%(before)s<a href="%(scheme)s%(body)s" target="_blank">'
+            '%(raw_scheme)s%(body)s</a>' %
+            {'before': x.group(1), 'raw_scheme': x.group(2),
+             'body': x.group(3), 'scheme': correct_scheme(x.group(2))},
+            message)
 
         return message
 
@@ -266,64 +329,14 @@ def convert(lines, filename_noext, template_dir=None):
 
 
 def convert_file(input_file, output_dir, template_dir=None):
-    filename_noext = re.sub("\.dat\.gz$|\.dat$", "",
-                            os.path.basename(input_file))
-    lines = open_file(input_file).readlines()
-    output = convert(lines, filename_noext, template_dir)
-
-    if output is None:
-        logging.error("Could not parse file: %s" % input_file)
-        return False
-
-    output_file = os.path.join(output_dir, filename_noext + ".html")
-    if template_dir == "*text*":
-        output_file = os.path.join(output_dir, filename_noext + ".txt")
-    if os.path.exists(output_file):
-        logging.warning("%s already exists. Overwriting ..." %
-                        output_file)
-
-    logging.info("Generating %s" % output_file)
-    try:
-        if output_dir == "-":
-            output_file = "stdout"
-            sys.stdout.write(output)
-        else:
-            f = open(output_file, "w")
-            f.write(output)
-            f.close()
-    except:
-        logging.error("Failed to write file: %s" % output_file)
-        return False
-
-    return True
+    dat2html = Dat2Html(template_dir)
+    return dat2html.convert_file(input_file, output_dir)
 
 
 def convert_files(input_files, output_dir, template_dir=None,
                   index=False, subject=False):
-    filenames = get_filenames(input_files)
-
-    if len(filenames) <= 0:
-        logging.error("No input files")
-        sys.exit(2)
-
-    if output_dir != "-" and not os.path.exists(output_dir):
-        logging.info("Creating directory ...")
-        try:
-            os.makedirs(output_dir)
-        except OSError, (errorno, strerror):
-            logging.error("Could not create %s: %s" % (output_dir, strerror))
-            sys.exit(2)
-    if output_dir != "-" and not os.access(output_dir, os.W_OK):
-        logging.error("Could not open %s: permission denied" % output_dir)
-        sys.exit(2)
-
-    for filename in filenames:
-        convert_file(filename, output_dir, template_dir)
-
-    if index:
-        make_index(filenames, output_dir)
-    if subject:
-        make_subject(filenames, output_dir)
+    dat2html = Dat2Html(template_dir)
+    return dat2html.convert_files(input_files, output_dir, index, subject)
 
 
 def get_filenames(input_files):
